# Licensed under a 3-clause BSD style license - see LICENSE.rst

"""
IO Interface for Reading CCSDS Data in Python.
"""

# For egg_info test builds to pass, put package imports here.

from .interface import (FixedLength, PacketField)




